@@ -245,15 +245,6 @@ static bytecodes compile_sub_expression (tree_node expr);
 static bytecodes compile_sub_expression_and_jump (tree_node expr,
 						  int branchLen);
 
-/* Scans the constants that are referenced by the current method.  If
-   one is found that is equal to CONSTEXPR, the index of that constant
-   is returned.  Otherwise, the constant is turned into a constant
-   object, added to the constants of the method, and the new index is
-   returned.
-
-   For small integers (between -128 and 255) return -1.  */
-static int add_constant (tree_node constExpr);
-
 /* Like add_constant, this routine adds "selector" to the set of
    selectors for the current method, and returns the index of that
    selector.  If the selector already existed, its index is returned.
@@ -1038,17 +1029,42 @@ compile_variable (tree_node varName)
 void
 compile_constant (tree_node constExpr)
 {
-  int index = add_constant (constExpr);
-  int value;
+  int index = -1;
+  OOP constantOOP;
+  OOP *lit;
+
+  /* Scan the current literal frame, looking for a constant equal
+     to the one that is being compiled.  */
+  for (lit = literal_vec; lit < literal_vec_curr; lit++)
+    if (equal_constant (*lit, constExpr))
+      {
+	index = lit - literal_vec;
+	break;
+      }
+
+  /* If not found, check if it can be compiled with a PUSH_INTEGER
+     bytecode, or add it to the literals.  */
+  if (index == -1)
+    {
+      constantOOP = make_constant_oop (constExpr);
+      if (IS_INT (constantOOP))
+        {
+          long intVal = TO_INT (constantOOP);
+          if (intVal >= -128 && intVal < 255)
+            {
+	      INCR_STACK_DEPTH ();
+	      compile_push_int_constant(intVal);
+              return;
+            }
+        }
+      else
+        MAKE_OOP_READONLY (constantOOP, true);
+
+      index = add_literal (constantOOP);
+    }
 
   INCR_STACK_DEPTH ();
-  if COMMON (index != -1)
-      compile_push_literal (index);
-  else
-    {
-      value = constExpr->v_const.val.iVal;
-      compile_push_int_constant(value);
-    }
+  compile_push_literal (index);
 }
 
 void
@@ -2094,30 +2110,6 @@ is_super (tree_node expr)
 }
 
 
-int
-add_constant (tree_node constExpr)
-{
-  long intVal;
-  OOP constantOOP;
-  OOP *lit;
-
-  for (lit = literal_vec; lit < literal_vec_curr; lit++)
-    if (equal_constant (*lit, constExpr))
-      return (lit - literal_vec);
-
-  constantOOP = make_constant_oop (constExpr);
-  if (IS_INT (constantOOP))
-    {
-      intVal = TO_INT (constantOOP);
-      if (intVal >= -128 && intVal <= 255)
-	return (-1);
-    }
-  else
-    MAKE_OOP_READONLY (constantOOP, true);
-
-  return (add_literal (constantOOP));
-}
-
 mst_Boolean
 equal_constant (OOP oop,
 		tree_node constExpr)
