@@ -46,19 +46,6 @@
 #define TIMER_REAL       SIGALRM
 #define TIMER_PROCESS    SIGVTALRM
 
-#ifndef  O_NONBLOCK
-#ifdef  O_NDELAY
-#define O_NONBLOCK O_NDELAY
-#else
-#ifdef  FNDELAY
-#define O_NONBLOCK FNDELAY
-#else
-#warning Non-blocking I/O could not be enabled
-#define O_NONBLOCK 0
-#endif
-#endif
-#endif
-
 #ifndef HAVE_SYS_MMAN_H
 #define PROT_READ       0x1             /* Page can be read.  */
 #define PROT_WRITE      0x2             /* Page can be written.  */
