@@ -47,14 +47,16 @@ typedef struct method_entry
 }
 method_entry;
 
-extern PTR get_native_code (OOP methodOOP, OOP receiverClass);
+extern void _gst_reset_inline_caches ();
 
-extern PTR map_virtual_ip (OOP methodOOP, OOP receiverClass, int ip);
+extern PTR _gst_get_native_code (OOP methodOOP, OOP receiverClass);
 
-extern void free_released_native_code (void);
-extern void release_native_code (OOP methodOOP);
-extern void discard_native_code (OOP methodOOP);
-extern void init_translator (void);
+extern PTR _gst_map_virtual_ip (OOP methodOOP, OOP receiverClass, int ip);
+
+extern void _gst_free_released_native_code (void);
+extern void _gst_release_native_code (OOP methodOOP);
+extern void _gst_discard_native_code (OOP methodOOP);
+extern void _gst_init_translator (void);
 
 extern PTR (*_gst_run_native_code) ();
 
